@@ -8,9 +8,9 @@ module ViewComponent
       prepend_view_path File.expand_path("../../../views", __dir__)
 
       around_action :set_locale, only: :previews
-      before_action :require_local!, unless: :show_previews?
+      before_action :require_local!, unless: :previews_enabled?
 
-      content_security_policy(false) if respond_to?(:content_security_policy)
+      content_security_policy(false)
 
       # Including helpers here ensures that we're loading the
       # latest version of helpers if code-reloading is enabled
@@ -47,12 +47,12 @@ module ViewComponent
 
     # :doc:
     def default_preview_layout
-      ViewComponent::Base.config.default_preview_layout
+      ViewComponent::Base.config.previews.default_layout
     end
 
     # :doc:
-    def show_previews?
-      ViewComponent::Base.config.show_previews
+    def previews_enabled?
+      ViewComponent::Base.config.previews.enabled
     end
 
     # :doc:
@@ -64,7 +64,11 @@ module ViewComponent
       if preview
         @preview = ViewComponent::Preview.find(preview)
       else
+        # TODO: This branch is covered in #test_returns_404_when_preview_does_not_exist,
+        # but Simplecov doesn't always mark it as covered.
+        # :nocov:
         raise AbstractController::ActionNotFound, "Component preview '#{params[:path]}' not found."
+        # :nocov:
       end
     end
 
@@ -95,7 +99,7 @@ module ViewComponent
     end
 
     def prepend_preview_examples_view_path
-      prepend_view_path(ViewComponent::Base.preview_paths)
+      prepend_view_path(ViewComponent::Base.previews.paths)
     end
   end
 end
