@@ -1,7 +1,7 @@
 # frozen_string_literal: true
 
 require "action_view"
-require "active_support/configurable"
+require "action_view/base"
 require "view_component/collection"
 require "view_component/compile_cache"
 require "view_component/compiler"
@@ -9,13 +9,27 @@ require "view_component/config"
 require "view_component/errors"
 require "view_component/inline_template"
 require "view_component/preview"
+require "view_component/request_details"
 require "view_component/slotable"
+require "view_component/template"
 require "view_component/translatable"
 require "view_component/with_content_helper"
-require "view_component/use_helpers"
+
+module ActionView
+  class OutputBuffer
+    def with_buffer(buf = nil)
+      new_buffer = buf || +""
+      old_buffer, @raw_buffer = @raw_buffer, new_buffer
+      yield
+      new_buffer
+    ensure
+      @raw_buffer = old_buffer
+    end
+  end
+end
 
 module ViewComponent
-  class Base < ActionView::Base
+  class Base
     class << self
       delegate(*ViewComponent::Config.defaults.keys, to: :config)
 
@@ -23,29 +37,42 @@ module ViewComponent
       #
       # @return [ActiveSupport::OrderedOptions]
       def config
+        module_parents.each do |module_parent|
+          next unless module_parent.respond_to?(:config)
+          module_parent_config = module_parent.config.try(:view_component)
+          return module_parent_config if module_parent_config
+        end
         ViewComponent::Config.current
       end
     end
 
+    include ActionView::Helpers
+    include Rails.application.routes.url_helpers if defined?(Rails.application.routes)
+    include ERB::Escape
+    include ActiveSupport::CoreExt::ERBUtil
+
     include ViewComponent::InlineTemplate
-    include ViewComponent::UseHelpers
     include ViewComponent::Slotable
     include ViewComponent::Translatable
     include ViewComponent::WithContentHelper
 
-    RESERVED_PARAMETER = :content
-
     # For CSRF authenticity tokens in forms
     delegate :form_authenticity_token, :protect_against_forgery?, :config, to: :helpers
 
+    # HTML construction methods
+    delegate :output_buffer, :lookup_context, :view_renderer, :view_flow, to: :helpers
+
+    # For Turbo::StreamsHelper
+    delegate :formats, :formats=, to: :helpers
+
     # For Content Security Policy nonces
     delegate :content_security_policy_nonce, to: :helpers
 
     # Config option that strips trailing whitespace in templates before compiling them.
-    class_attribute :__vc_strip_trailing_whitespace, instance_accessor: false, instance_predicate: false
-    self.__vc_strip_trailing_whitespace = false # class_attribute:default doesn't work until Rails 5.2
+    class_attribute :__vc_strip_trailing_whitespace, instance_accessor: false, instance_predicate: false, default: false
 
     attr_accessor :__vc_original_view_context
+    attr_reader :current_template
 
     # Components render in their own view context. Helpers and other functionality
     # require a reference to the original Rails view context, an instance of
@@ -57,7 +84,14 @@ module ViewComponent
     # @param view_context [ActionView::Base] The original view context.
     # @return [void]
     def set_original_view_context(view_context)
-      self.__vc_original_view_context = view_context
+      # noop
+    end
+
+    using RequestDetails
+
+    # Including `Rails.application.routes.url_helpers` defines an initializer that accepts (...),
+    # so we have to define our own empty initializer to overwrite it.
+    def initialize
     end
 
     # Entrypoint for rendering components.
@@ -69,31 +103,28 @@ module ViewComponent
     #
     # @return [String]
     def render_in(view_context, &block)
-      self.class.compile(raise_errors: true)
+      self.class.__vc_compile(raise_errors: true)
 
       @view_context = view_context
+      @old_virtual_path = view_context.instance_variable_get(:@virtual_path)
       self.__vc_original_view_context ||= view_context
 
-      @output_buffer = ActionView::OutputBuffer.new
+      @output_buffer = view_context.output_buffer
 
       @lookup_context ||= view_context.lookup_context
 
-      # required for path helpers in older Rails versions
-      @view_renderer ||= view_context.view_renderer
-
       # For content_for
       @view_flow ||= view_context.view_flow
 
       # For i18n
       @virtual_path ||= virtual_path
 
-      # For template variants (+phone, +desktop, etc.)
-      @__vc_variant ||= @lookup_context.variants.first
+      # Describes the inferred request constraints (locales, formats, variants)
+      @__vc_requested_details ||= @lookup_context.vc_requested_details
 
       # For caching, such as #cache_if
       @current_template = nil unless defined?(@current_template)
       old_current_template = @current_template
-      @current_template = self
 
       if block && defined?(@__vc_content_set_by_with_content)
         raise DuplicateContentError.new(self.class.name)
@@ -101,22 +132,38 @@ module ViewComponent
 
       @__vc_content_evaluated = false
       @__vc_render_in_block = block
+      @view_context.instance_variable_set(:@virtual_path, virtual_path)
 
       before_render
 
       if render?
-        # Avoid allocating new string when output_preamble and output_postamble are blank
-        rendered_template = safe_render_template_for(@__vc_variant).to_s
+        value = nil
 
-        if output_preamble.blank? && output_postamble.blank?
-          rendered_template
-        else
-          safe_output_preamble + rendered_template + safe_output_postamble
+        @output_buffer.with_buffer do
+          rendered_template =
+            around_render do
+              render_template_for(@__vc_requested_details).to_s
+            end
+
+          # Avoid allocating new string when output_preamble and output_postamble are blank
+          value = if output_preamble.blank? && output_postamble.blank?
+            rendered_template
+          else
+            __vc_safe_output_preamble + rendered_template + __vc_safe_output_postamble
+          end
         end
+
+        if ActionView::Base.annotate_rendered_view_with_filenames && current_template.inline_call? && request&.format == :html
+          identifier = defined?(Rails.root) ? self.class.identifier.sub("#{Rails.root}/", "") : self.class.identifier
+          value = "<!-- BEGIN #{identifier} -->".html_safe + value + "<!-- END #{identifier} -->".html_safe
+        end
+
+        value
       else
         ""
       end
     ensure
+      view_context.instance_variable_set(:@virtual_path, @old_virtual_path)
       @current_template = old_current_template
     end
 
@@ -153,7 +200,7 @@ module ViewComponent
         target_render = self.class.instance_variable_get(:@__vc_ancestor_calls)[@__vc_parent_render_level]
         @__vc_parent_render_level += 1
 
-        target_render.bind_call(self, @__vc_variant)
+        target_render.bind_call(self, @__vc_requested_details)
       ensure
         @__vc_parent_render_level -= 1
       end
@@ -181,6 +228,14 @@ module ViewComponent
       # noop
     end
 
+    # Called around rendering the component. Override to wrap the rendering of a
+    # component in custom instrumentation, etc.
+    #
+    # @return [void]
+    def around_render
+      yield
+    end
+
     # Override to determine whether the ViewComponent should render.
     #
     # @return [Boolean]
@@ -188,23 +243,37 @@ module ViewComponent
       true
     end
 
-    # @private
-    def initialize(*)
-    end
-
     # Re-use original view_context if we're not rendering a component.
     #
-    # This prevents an exception when rendering a partial inside of a component that has also been rendered outside
-    # of the component. This is due to the partials compiled template method existing in the parent `view_context`,
-    #  and not the component's `view_context`.
+    # As of v4, ViewComponent::Base re-uses the existing view context created
+    # by ActionView, meaning the current view context and the original view
+    # context are the same object. set_original_view_context is still called
+    # to maintain backwards compatibility.
     #
     # @private
     def render(options = {}, args = {}, &block)
       if options.respond_to?(:set_original_view_context)
         options.set_original_view_context(self.__vc_original_view_context)
-        super
+
+        # We assume options is a component, so there's no need to evaluate the
+        # block in the view context as we do below.
+        @view_context.render(options, args, &block)
+      elsif block
+        __vc_original_view_context.render(options, args) do
+          # capture the block output in the view context of the component
+          output = capture(&block)
+
+          # Partials are rendered to their own buffer and do not append to the
+          # original @output_buffer we retain a reference to in #render_in. This
+          # is a problem since the block passed to us here in the #render method
+          # is evaluated within the context of ViewComponent::Base, and thus
+          # appends to the original @output_buffer. To avoid this, we evaluate the
+          # block in the view context instead, which will append to the output buffer
+          # created for the partial.
+          __vc_original_view_context.capture { output }
+        end
       else
-        __vc_original_view_context.render(options, args, &block)
+        __vc_original_view_context.render(options, args)
       end
     end
 
@@ -235,7 +304,7 @@ module ViewComponent
       @__vc_helpers ||= __vc_original_view_context || controller.view_context
     end
 
-    if ::Rails.env.development? || ::Rails.env.test?
+    if defined?(Rails.env) && (::Rails.env.development? || ::Rails.env.test?)
       # @private
       def method_missing(method_name, *args) # rubocop:disable Style/MissingRespondToMissing
         super
@@ -244,7 +313,7 @@ module ViewComponent
         raise e, <<~MESSAGE.chomp if view_context && e.is_a?(NameError) && helpers.respond_to?(method_name)
           #{e.message}
 
-          You may be trying to call a method provided as a view helper. Did you mean `helpers.#{method_name}'?
+          You may be trying to call a method provided as a view helper. Did you mean `helpers.#{method_name}`?
         MESSAGE
 
         raise
@@ -264,11 +333,12 @@ module ViewComponent
       []
     end
 
-    # For caching, such as #cache_if
-    #
-    # @private
-    def format
-      @__vc_variant if defined?(@__vc_variant)
+    if defined?(Rails::VERSION) && Rails::VERSION::MAJOR == 7 && Rails::VERSION::MINOR == 1
+      # Rails expects us to define `format` on all renderables,
+      # but we do not know the `format` of a ViewComponent until runtime.
+      def format
+        nil
+      end
     end
 
     # The current request. Use sparingly as doing so introduces coupling that
@@ -276,7 +346,13 @@ module ViewComponent
     #
     # @return [ActionDispatch::Request]
     def request
-      @request ||= controller.request if controller.respond_to?(:request)
+      __vc_request
+    end
+
+    # @private
+    def __vc_request
+      # The current request (if present, as mailers/jobs/etc do not have a request)
+      @__vc_request ||= controller.request if controller.respond_to?(:request)
     end
 
     # The content passed to the component instance as a block.
@@ -288,7 +364,9 @@ module ViewComponent
 
       @__vc_content =
         if __vc_render_in_block_provided?
-          view_context.capture(self, &@__vc_render_in_block)
+          with_captured_virtual_path(@old_virtual_path) do
+            view_context.capture(self, &@__vc_render_in_block)
+          end
         elsif __vc_content_set_by_with_content_defined?
           @__vc_content_set_by_with_content
         end
@@ -301,6 +379,17 @@ module ViewComponent
       __vc_render_in_block_provided? || __vc_content_set_by_with_content_defined?
     end
 
+    # @private
+    # Temporarily sets the virtual path to the captured value, then restores it.
+    # This ensures translations and other path-dependent code execute with the correct scope.
+    def with_captured_virtual_path(captured_path)
+      old_virtual_path = @view_context.instance_variable_get(:@virtual_path)
+      @view_context.instance_variable_set(:@virtual_path, captured_path)
+      yield
+    ensure
+      @view_context.instance_variable_set(:@virtual_path, old_virtual_path)
+    end
+
     private
 
     attr_reader :view_context
@@ -313,12 +402,8 @@ module ViewComponent
       defined?(@__vc_content_set_by_with_content)
     end
 
-    def content_evaluated?
-      defined?(@__vc_content_evaluated) && @__vc_content_evaluated
-    end
-
-    def maybe_escape_html(text)
-      return text if request && !request.format.html?
+    def __vc_maybe_escape_html(text)
+      return text if @current_template && !@current_template.html?
       return text if text.blank?
 
       if text.html_safe?
@@ -329,68 +414,18 @@ module ViewComponent
       end
     end
 
-    def safe_render_template_for(variant)
-      if compiler.renders_template_for_variant?(variant)
-        render_template_for(variant)
-      else
-        maybe_escape_html(render_template_for(variant)) do
-          Kernel.warn("WARNING: The #{self.class} component rendered HTML-unsafe output. The output will be automatically escaped, but you may want to investigate.")
-        end
-      end
-    end
-
-    def safe_output_preamble
-      maybe_escape_html(output_preamble) do
+    def __vc_safe_output_preamble
+      __vc_maybe_escape_html(output_preamble) do
         Kernel.warn("WARNING: The #{self.class} component was provided an HTML-unsafe preamble. The preamble will be automatically escaped, but you may want to investigate.")
       end
     end
 
-    def safe_output_postamble
-      maybe_escape_html(output_postamble) do
+    def __vc_safe_output_postamble
+      __vc_maybe_escape_html(output_postamble) do
         Kernel.warn("WARNING: The #{self.class} component was provided an HTML-unsafe postamble. The postamble will be automatically escaped, but you may want to investigate.")
       end
     end
 
-    def compiler
-      @compiler ||= self.class.compiler
-    end
-
-    # Set the controller used for testing components:
-    #
-    # ```ruby
-    # config.view_component.test_controller = "MyTestController"
-    # ```
-    #
-    # Defaults to `nil`. If this is falsy, `"ApplicationController"` is used. Can also be
-    # configured on a per-test basis using `with_controller_class`.
-    #
-
-    # Set if render monkey patches should be included or not in Rails <6.1:
-    #
-    # ```ruby
-    # config.view_component.render_monkey_patch_enabled = false
-    # ```
-    #
-
-    # Path for component files
-    #
-    # ```ruby
-    # config.view_component.view_component_path = "app/my_components"
-    # ```
-    #
-    # Defaults to `nil`. If this is falsy, `app/components` is used.
-    #
-
-    # Parent class for generated components
-    #
-    # ```ruby
-    # config.view_component.component_parent_class = "MyBaseComponent"
-    # ```
-    #
-    # Defaults to nil. If this is falsy, generators will use
-    # "ApplicationComponent" if defined, "ViewComponent::Base" otherwise.
-    #
-
     # Configuration for generators.
     #
     # All options under this namespace default to `false` unless otherwise
@@ -412,6 +447,14 @@ module ViewComponent
     # config.view_component.generate.stimulus_controller = true
     # ```
     #
+    # #### `#typescript`
+    #
+    # Generate TypeScript files instead of JavaScript files:
+    #
+    # ```ruby
+    # config.view_component.generate.typescript = true
+    # ```
+    #
     # #### #locale
     #
     # Always generate translations file alongside the component:
@@ -440,10 +483,30 @@ module ViewComponent
     # ```
     #
     #  Defaults to `false`.
+    #
+    # #### ßparent_class
+    #
+    # Parent class for generated components
+    #
+    # ```ruby
+    # config.view_component.generate.parent_class = "MyBaseComponent"
+    # ```
+    #
+    # Defaults to nil. If this is falsy, generators will use
+    # "ApplicationComponent" if defined, "ViewComponent::Base" otherwise.
+    #
 
     class << self
+      # The file path of the component Ruby file.
+      #
+      # @return [String]
+      attr_reader :identifier
+
       # @private
-      attr_accessor :source_location, :virtual_path
+      attr_writer :identifier
+
+      # @private
+      attr_accessor :virtual_path
 
       # Find sidecar files for the given extensions.
       #
@@ -453,13 +516,13 @@ module ViewComponent
       # For example, one might collect sidecar CSS files that need to be compiled.
       # @param extensions [Array<String>] Extensions of which to return matching sidecar files.
       def sidecar_files(extensions)
-        return [] unless source_location
+        return [] unless identifier
 
         extensions = extensions.join(",")
 
         # view files in a directory named like the component
-        directory = File.dirname(source_location)
-        filename = File.basename(source_location, ".rb")
+        directory = File.dirname(identifier)
+        filename = File.basename(identifier, ".rb")
         component_name = name.demodulize.underscore
 
         # Add support for nested components defined in the same file.
@@ -484,7 +547,7 @@ module ViewComponent
 
         sidecar_directory_files = Dir["#{directory}/#{component_name}/#{filename}.*{#{extensions}}"]
 
-        (sidecar_files - [source_location] + sidecar_directory_files + nested_component_files).uniq
+        (sidecar_files - [identifier] + sidecar_directory_files + nested_component_files).uniq
       end
 
       # Render a component for each element in a collection ([documentation](/guide/collections)):
@@ -494,64 +557,51 @@ module ViewComponent
       # ```
       #
       # @param collection [Enumerable] A list of items to pass the ViewComponent one at a time.
+      # @param spacer_component [ViewComponent::Base] Component instance to be rendered between items.
       # @param args [Arguments] Arguments to pass to the ViewComponent every time.
-      def with_collection(collection, **args)
-        Collection.new(self, collection, **args)
+      def with_collection(collection, spacer_component: nil, **args)
+        Collection.new(self, collection, spacer_component, **args)
       end
 
-      # Provide identifier for ActionView template annotations
-      #
       # @private
-      def short_identifier
-        @short_identifier ||= defined?(Rails.root) ? source_location.sub("#{Rails.root}/", "") : source_location
+      def __vc_compile(raise_errors: false, force: false)
+        __vc_compiler.compile(raise_errors: raise_errors, force: force)
       end
 
       # @private
       def inherited(child)
         # Compile so child will inherit compiled `call_*` template methods that
         # `compile` defines
-        compile
+        __vc_compile
 
         # Give the child its own personal #render_template_for to protect against the case when
         # eager loading is disabled and the parent component is rendered before the child. In
         # such a scenario, the parent will override ViewComponent::Base#render_template_for,
         # meaning it will not be called for any children and thus not compile their templates.
-        if !child.instance_methods(false).include?(:render_template_for) && !child.compiled?
+        if !child.method_defined?(:render_template_for, false) && !child.__vc_compiled?
           child.class_eval <<~RUBY, __FILE__, __LINE__ + 1
-            def render_template_for(variant = nil)
+            def render_template_for(requested_details)
               # Force compilation here so the compiler always redefines render_template_for.
               # This is mostly a safeguard to prevent infinite recursion.
-              self.class.compile(raise_errors: true, force: true)
-              # .compile replaces this method; call the new one
-              render_template_for(variant)
+              self.class.__vc_compile(raise_errors: true, force: true)
+              # .__vc_compile replaces this method; call the new one
+              render_template_for(requested_details)
             end
           RUBY
         end
 
-        # If Rails application is loaded, add application url_helpers to the component context
-        # we need to check this to use this gem as a dependency
-        if defined?(Rails) && Rails.application && !(child < Rails.application.routes.url_helpers)
-          child.include Rails.application.routes.url_helpers
-        end
-
         # Derive the source location of the component Ruby file from the call stack.
         # We need to ignore `inherited` frames here as they indicate that `inherited`
         # has been re-defined by the consuming application, likely in ApplicationComponent.
         # We use `base_label` method here instead of `label` to avoid cases where the method
         # owner is included in a prefix like `ApplicationComponent.inherited`.
-        child.source_location = caller_locations(1, 10).reject { |l| l.base_label == "inherited" }[0].path
-
-        # If Rails application is loaded, removes the first part of the path and the extension.
-        if defined?(Rails) && Rails.application
-          child.virtual_path = child.source_location.gsub(
-            /(.*#{Regexp.quote(ViewComponent::Base.config.view_component_path)})|(\.rb)/, ""
-          )
-        end
+        child.identifier = caller_locations(1, 10).reject { |l| l.base_label == "inherited" }[0].path
+        child.virtual_path = child.name&.underscore
 
         # Set collection parameter to the extended component
-        child.with_collection_parameter provided_collection_parameter
+        child.with_collection_parameter(__vc_provided_collection_parameter)
 
-        if instance_methods(false).include?(:render_template_for)
+        if method_defined?(:render_template_for, false)
           vc_ancestor_calls = defined?(@__vc_ancestor_calls) ? @__vc_ancestor_calls.dup : []
 
           vc_ancestor_calls.unshift(instance_method(:render_template_for))
@@ -562,45 +612,30 @@ module ViewComponent
       end
 
       # @private
-      def compiled?
-        compiler.compiled?
+      def __vc_compiled?
+        __vc_compiler.compiled?
       end
 
-      # @private
-      def ensure_compiled
-        compile unless compiled?
-      end
-
-      # Compile templates to instance methods, assuming they haven't been compiled already.
+      # Hook called by the compiler after a component is compiled.
       #
-      # Do as much work as possible in this step, as doing so reduces the amount
-      # of work done each time a component is rendered.
-      # @private
-      def compile(raise_errors: false, force: false)
-        compiler.compile(raise_errors: raise_errors, force: force)
+      # Extensions can override this class method to run logic after
+      # compilation (e.g., generate helpers, register metadata, etc.).
+      #
+      # By default, this is a no-op.
+      def after_compile
+        # no-op by default
       end
 
       # @private
-      def compiler
+      def __vc_ensure_compiled
+        __vc_compile unless __vc_compiled?
+      end
+
+      # @private
+      def __vc_compiler
         @__vc_compiler ||= Compiler.new(self)
       end
 
-      # we'll eventually want to update this to support other types
-      # @private
-      def type
-        "text/html"
-      end
-
-      # @private
-      def format
-        :html
-      end
-
-      # @private
-      def identifier
-        source_location
-      end
-
       # Set the parameter name used when rendering elements of a collection ([documentation](/guide/collections)):
       #
       # ```ruby
@@ -609,8 +644,8 @@ module ViewComponent
       #
       # @param parameter [Symbol] The parameter name used when rendering elements of a collection.
       def with_collection_parameter(parameter)
-        @provided_collection_parameter = parameter
-        @initialize_parameters = nil
+        @__vc_provided_collection_parameter = parameter
+        @__vc_initialize_parameters = nil
       end
 
       # Strips trailing whitespace from templates before compiling them.
@@ -638,19 +673,12 @@ module ViewComponent
       # validate that the default parameter name
       # is accepted, as support for collection
       # rendering is optional.
-      # @private TODO: add documentation
-      def validate_collection_parameter!(validate_default: false)
-        parameter = validate_default ? collection_parameter : provided_collection_parameter
+      # @private
+      def __vc_validate_collection_parameter!(validate_default: false)
+        parameter = validate_default ? __vc_collection_parameter : __vc_provided_collection_parameter
 
         return unless parameter
-        return if initialize_parameter_names.include?(parameter) || splatted_keyword_argument_present?
-
-        # If Ruby can't parse the component class, then the initialize
-        # parameters will be empty and ViewComponent will not be able to render
-        # the component.
-        if initialize_parameters.empty?
-          raise EmptyOrInvalidInitializerError.new(name, parameter)
-        end
+        return if __vc_initialize_parameter_names.include?(parameter) || __vc_splatted_keyword_argument_present?
 
         raise MissingCollectionArgumentError.new(name, parameter)
       end
@@ -658,59 +686,59 @@ module ViewComponent
       # Ensure the component initializer doesn't define
       # invalid parameters that could override the framework's
       # methods.
-      # @private TODO: add documentation
-      def validate_initialization_parameters!
-        return unless initialize_parameter_names.include?(RESERVED_PARAMETER)
+      # @private
+      def __vc_validate_initialization_parameters!
+        return unless __vc_initialize_parameter_names.include?(:content)
 
-        raise ReservedParameterError.new(name, RESERVED_PARAMETER)
+        raise ReservedParameterError.new(name, :content)
       end
 
       # @private
-      def collection_parameter
-        provided_collection_parameter || name && name.demodulize.underscore.chomp("_component").to_sym
+      def __vc_collection_parameter
+        @__vc_provided_collection_parameter ||= name && name.demodulize.underscore.chomp("_component").to_sym
       end
 
       # @private
-      def collection_counter_parameter
-        :"#{collection_parameter}_counter"
+      def __vc_collection_counter_parameter
+        @__vc_collection_counter_parameter ||= :"#{__vc_collection_parameter}_counter"
       end
 
       # @private
-      def counter_argument_present?
-        initialize_parameter_names.include?(collection_counter_parameter)
+      def __vc_counter_argument_present?
+        __vc_initialize_parameter_names.include?(__vc_collection_counter_parameter)
       end
 
       # @private
-      def collection_iteration_parameter
-        :"#{collection_parameter}_iteration"
+      def __vc_collection_iteration_parameter
+        @__vc_collection_iteration_parameter ||= :"#{__vc_collection_parameter}_iteration"
       end
 
       # @private
-      def iteration_argument_present?
-        initialize_parameter_names.include?(collection_iteration_parameter)
+      def __vc_iteration_argument_present?
+        __vc_initialize_parameter_names.include?(__vc_collection_iteration_parameter)
       end
 
       private
 
-      def splatted_keyword_argument_present?
-        initialize_parameters.flatten.include?(:keyrest) &&
-          !initialize_parameters.include?([:keyrest, :**]) # Un-named splatted keyword args don't count!
+      def __vc_splatted_keyword_argument_present?
+        __vc_initialize_parameters.flatten.include?(:keyrest)
       end
 
-      def initialize_parameter_names
-        return attribute_names.map(&:to_sym) if respond_to?(:attribute_names)
-
-        return attribute_types.keys.map(&:to_sym) if Rails::VERSION::MAJOR <= 5 && respond_to?(:attribute_types)
-
-        initialize_parameters.map(&:last)
+      def __vc_initialize_parameter_names
+        @__vc_initialize_parameter_names ||=
+          if respond_to?(:attribute_names)
+            attribute_names.map(&:to_sym)
+          else
+            __vc_initialize_parameters.map(&:last)
+          end
       end
 
-      def initialize_parameters
-        @initialize_parameters ||= instance_method(:initialize).parameters
+      def __vc_initialize_parameters
+        @__vc_initialize_parameters ||= instance_method(:initialize).parameters
       end
 
-      def provided_collection_parameter
-        @provided_collection_parameter ||= nil
+      def __vc_provided_collection_parameter
+        @__vc_provided_collection_parameter ||= nil
       end
     end
 
