@@ -1,34 +1,41 @@
 # frozen_string_literal: true
 
-require "action_view/renderer/collection_renderer" if Rails.version.to_f >= 6.1
+require "action_view/renderer/collection_renderer"
 
 module ViewComponent
   class Collection
     include Enumerable
+
     attr_reader :component
 
-    delegate :format, to: :component
     delegate :size, to: :@collection
 
-    attr_accessor :__vc_original_view_context
-
-    def set_original_view_context(view_context)
-      self.__vc_original_view_context = view_context
-    end
-
     def render_in(view_context, &block)
       components.map do |component|
-        component.set_original_view_context(__vc_original_view_context)
         component.render_in(view_context, &block)
-      end.join.html_safe
+      end.join(rendered_spacer(view_context)).html_safe
     end
 
+    def each(&block)
+      components.each(&block)
+    end
+
+    if defined?(Rails::VERSION) && Rails::VERSION::MAJOR == 7 && Rails::VERSION::MINOR == 1
+      # Rails expects us to define `format` on all renderables,
+      # but we do not know the `format` of a ViewComponent until runtime.
+      def format
+        nil
+      end
+    end
+
+    private
+
     def components
       return @components if defined? @components
 
       iterator = ActionView::PartialIteration.new(@collection.size)
 
-      component.validate_collection_parameter!(validate_default: true)
+      component.__vc_validate_collection_parameter!(validate_default: true)
 
       @components = @collection.map do |item|
         component.new(**component_options(item, iterator)).tap do |component|
@@ -37,15 +44,10 @@ module ViewComponent
       end
     end
 
-    def each(&block)
-      components.each(&block)
-    end
-
-    private
-
-    def initialize(component, object, **options)
+    def initialize(component, object, spacer_component, **options)
       @component = component
       @collection = collection_variable(object || [])
+      @spacer_component = spacer_component
       @options = options
     end
 
@@ -58,11 +60,19 @@ module ViewComponent
     end
 
     def component_options(item, iterator)
-      item_options = {component.collection_parameter => item}
-      item_options[component.collection_counter_parameter] = iterator.index if component.counter_argument_present?
-      item_options[component.collection_iteration_parameter] = iterator.dup if component.iteration_argument_present?
+      item_options = {component.__vc_collection_parameter => item}
+      item_options[component.__vc_collection_counter_parameter] = iterator.index if component.__vc_counter_argument_present?
+      item_options[component.__vc_collection_iteration_parameter] = iterator.dup if component.__vc_iteration_argument_present?
 
       @options.merge(item_options)
     end
+
+    def rendered_spacer(view_context)
+      if @spacer_component
+        @spacer_component.render_in(view_context)
+      else
+        ""
+      end
+    end
   end
 end
