@@ -5,12 +5,14 @@ require "active_support/notifications"
 module ViewComponent # :nodoc:
   module Instrumentation
     def self.included(mod)
-      mod.prepend(self) unless ancestors.include?(ViewComponent::Instrumentation)
+      mod.prepend(self) unless self <= ViewComponent::Instrumentation
     end
 
     def render_in(view_context, &block)
+      return super if !Rails.application.config.view_component.instrumentation_enabled.present?
+
       ActiveSupport::Notifications.instrument(
-        notification_name,
+        "render.view_component",
         {
           name: self.class.name,
           identifier: self.class.identifier
@@ -19,13 +21,5 @@ module ViewComponent # :nodoc:
         super
       end
     end
-
-    private
-
-    def notification_name
-      return "!render.view_component" if ViewComponent::Base.config.use_deprecated_instrumentation_name
-
-      "render.view_component"
-    end
   end
 end
