@@ -6,7 +6,7 @@ module ViewComponent
   class Slot
     include ViewComponent::WithContentHelper
 
-    attr_writer :__vc_component_instance, :__vc_content_block, :__vc_content
+    attr_writer :__vc_component_instance, :__vc_content_block, :__vc_content, :__vc_content_block_virtual_path
 
     def initialize(parent)
       @parent = parent
@@ -58,13 +58,7 @@ module ViewComponent
           if defined?(@__vc_content_block)
             # render_in is faster than `parent.render`
             @__vc_component_instance.render_in(view_context) do |*args|
-              return @__vc_content_block.call(*args) if @__vc_content_block&.source_location.nil?
-
-              block_context = @__vc_content_block.binding.receiver
-
-              if block_context.class < ActionView::Base
-                block_context.capture(*args, &@__vc_content_block)
-              else
+              @parent.with_captured_virtual_path(@__vc_content_block_virtual_path) do
                 @__vc_content_block.call(*args)
               end
             end
@@ -74,7 +68,9 @@ module ViewComponent
         elsif defined?(@__vc_content)
           @__vc_content
         elsif defined?(@__vc_content_block)
-          view_context.capture(&@__vc_content_block)
+          @parent.with_captured_virtual_path(@__vc_content_block_virtual_path) do
+            view_context.capture(&@__vc_content_block)
+          end
         elsif defined?(@__vc_content_set_by_with_content)
           @__vc_content_set_by_with_content
         end
@@ -101,15 +97,12 @@ module ViewComponent
     #   end
     # end
     #
-    def method_missing(symbol, *args, &block)
-      @__vc_component_instance.public_send(symbol, *args, &block)
+    def method_missing(symbol, *args, **kwargs, &block)
+      @__vc_component_instance.public_send(symbol, *args, **kwargs, &block)
     end
-    ruby2_keywords(:method_missing) if respond_to?(:ruby2_keywords, true)
 
     def html_safe?
-      # :nocov:
       to_s.html_safe?
-      # :nocov:
     end
 
     def respond_to_missing?(symbol, include_all = false)
