@@ -4,7 +4,6 @@ module ViewComponent
   module SystemTestHelpers
     include TestHelpers
 
-    #
     # Returns a block that can be used to visit the path of the inline rendered component.
     # @param fragment [Nokogiri::Fragment] The fragment returned from `render_inline`.
     # @param layout [String] The (optional) layout to use.
@@ -18,7 +17,7 @@ module ViewComponent
         file.write(vc_test_controller.render_to_string(html: fragment.to_html.html_safe, layout: layout))
         file.rewind
 
-        block.call("/_system_test_entrypoint?file=#{file.path.split("/").last}")
+        yield("/_system_test_entrypoint?file=#{file.path.split("/").last}")
       ensure
         file.unlink
       end
