@@ -14,18 +14,11 @@ module ViewComponent
       def refute_component_rendered
         assert_no_selector("body")
       end
-    rescue LoadError
-      # We don't have a test case for running an application without capybara installed.
-      # It's probably fine to leave this without coverage.
-      # :nocov:
-      if ENV["DEBUG"]
-        warn(
-          "WARNING in `ViewComponent::TestHelpers`: Add `capybara` " \
-          "to Gemfile to use Capybara assertions."
-        )
-      end
 
-      # :nocov:
+      def assert_component_rendered
+        assert_selector("body")
+      end
+    rescue LoadError # We don't have a test case for running an application without capybara installed.
     end
 
     # Returns the result of a render_inline call.
@@ -42,21 +35,32 @@ module ViewComponent
     # ```
     #
     # @param component [ViewComponent::Base, ViewComponent::Collection] The instance of the component to be rendered.
-    # @return [Nokogiri::HTML]
+    # @return [Nokogiri::HTML5]
     def render_inline(component, **args, &block)
       @page = nil
-      @rendered_content =
-        if Rails.version.to_f >= 6.1
-          vc_test_controller.view_context.render(component, args, &block)
+      @rendered_content = vc_test_view_context.render(component, args, &block)
 
-        # :nocov:
-        else
-          vc_test_controller.view_context.render_component(component, &block)
-        end
+      fragment = Nokogiri::HTML5.fragment(@rendered_content, context: "template")
+      @vc_test_view_context = nil
+      fragment
+    end
 
-      # :nocov:
+    # Returns the view context used to render components in tests. Note that the view context
+    # is reset after each call to `render_inline`.
+    #
+    # @return [ActionView::Base]
+    def vc_test_view_context
+      @vc_test_view_context ||= vc_test_controller.view_context
+    end
 
-      Nokogiri::HTML.fragment(@rendered_content)
+    # `JSON.parse`-d component output.
+    #
+    # ```ruby
+    # render_inline(MyJsonComponent.new)
+    # assert_equal(rendered_json["hello"], "world")
+    # ```
+    def rendered_json
+      JSON.parse(rendered_content)
     end
 
     # Render a preview inline. Internally sets `page` to be a `Capybara::Node::Simple`,
@@ -77,9 +81,9 @@ module ViewComponent
     # @param name [String] The name of the preview to be rendered.
     # @param from [ViewComponent::Preview] The class of the preview to be rendered.
     # @param params [Hash] Parameters to be passed to the preview.
-    # @return [Nokogiri::HTML]
+    # @return [Nokogiri::HTML5]
     def render_preview(name, from: __vc_test_helpers_preview_class, params: {})
-      previews_controller = __vc_test_helpers_build_controller(Rails.application.config.view_component.preview_controller.constantize)
+      previews_controller = __vc_test_helpers_build_controller(Rails.application.config.view_component.previews.controller.constantize)
 
       # From what I can tell, it's not possible to overwrite all request parameters
       # at once, so we set them individually here.
@@ -93,7 +97,7 @@ module ViewComponent
 
       @rendered_content = result
 
-      Nokogiri::HTML.fragment(@rendered_content)
+      Nokogiri::HTML5.fragment(@rendered_content)
     end
 
     # Execute the given block in the view context (using `instance_exec`).
@@ -107,12 +111,11 @@ module ViewComponent
     #
     # assert_text("Hello, World!")
     # ```
-    def render_in_view_context(*args, &block)
+    def render_in_view_context(...)
       @page = nil
-      @rendered_content = vc_test_controller.view_context.instance_exec(*args, &block)
-      Nokogiri::HTML.fragment(@rendered_content)
+      @rendered_content = vc_test_view_context.instance_exec(...)
+      Nokogiri::HTML5.fragment(@rendered_content)
     end
-    ruby2_keywords(:render_in_view_context) if respond_to?(:ruby2_keywords, true)
 
     # Set the Action Pack request variant for the given block:
     #
@@ -122,11 +125,11 @@ module ViewComponent
     # end
     # ```
     #
-    # @param variant [Symbol] The variant to be set for the provided block.
-    def with_variant(variant)
+    # @param variants [Symbol[]] The variants to be set for the provided block.
+    def with_variant(*variants)
       old_variants = vc_test_controller.view_context.lookup_context.variants
 
-      vc_test_controller.view_context.lookup_context.variants = variant
+      vc_test_controller.view_context.lookup_context.variants += variants
       yield
     ensure
       vc_test_controller.view_context.lookup_context.variants = old_variants
@@ -141,7 +144,7 @@ module ViewComponent
     # end
     # ```
     #
-    # @param klass [ActionController::Base] The controller to be used.
+    # @param klass [Class<ActionController::Base>] The controller to be used.
     def with_controller_class(klass)
       old_controller = defined?(@vc_test_controller) && @vc_test_controller
 
@@ -151,6 +154,24 @@ module ViewComponent
       @vc_test_controller = old_controller
     end
 
+    # Set format of the current request
+    #
+    # ```ruby
+    # with_format(:json) do
+    #   render_inline(MyComponent.new)
+    # end
+    # ```
+    #
+    # @param formats [Symbol[]] The format(s) to be set for the provided block.
+    def with_format(*formats)
+      old_formats = vc_test_controller.view_context.lookup_context.formats
+
+      vc_test_controller.view_context.lookup_context.formats = formats
+      yield
+    ensure
+      vc_test_controller.view_context.lookup_context.formats = old_formats
+    end
+
     # Set the URL of the current request (such as when using request-dependent path helpers):
     #
     # ```ruby
@@ -175,10 +196,19 @@ module ViewComponent
     # end
     # ```
     #
+    # To specify a protocol, pass the protocol param:
+    #
+    # ```ruby
+    # with_request_url("/users/42", protocol: :https) do
+    #   render_inline(MyComponent.new)
+    # end
+    # ```
+    #
     # @param full_path [String] The path to set for the current request.
     # @param host [String] The host to set for the current request.
     # @param method [String] The request method to set for the current request.
-    def with_request_url(full_path, host: nil, method: nil, format: :html)
+    # @param protocol [Symbol] The protocol to set for the current request (e.g., `:http` or `:https`).
+    def with_request_url(full_path, host: nil, method: nil, protocol: nil)
       old_request_host = vc_test_request.host
       old_request_method = vc_test_request.request_method
       old_request_path_info = vc_test_request.path_info
@@ -186,6 +216,7 @@ module ViewComponent
       old_request_query_parameters = vc_test_request.query_parameters
       old_request_query_string = vc_test_request.query_string
       old_request_format = vc_test_request.format.symbol
+      old_request_scheme = vc_test_request.scheme
       old_controller = defined?(@vc_test_controller) && @vc_test_controller
 
       path, query = full_path.split("?", 2)
@@ -193,16 +224,17 @@ module ViewComponent
       vc_test_request.instance_variable_set(:@original_fullpath, full_path)
       vc_test_request.host = host if host
       vc_test_request.request_method = method if method
+      vc_test_request.set_header(Rack::RACK_URL_SCHEME, protocol.to_s) if protocol
       vc_test_request.path_info = path
       vc_test_request.path_parameters = Rails.application.routes.recognize_path_with_request(vc_test_request, path, {})
       vc_test_request.set_header("action_dispatch.request.query_parameters",
         Rack::Utils.parse_nested_query(query).with_indifferent_access)
       vc_test_request.set_header(Rack::QUERY_STRING, query)
-      vc_test_request.format = format
       yield
     ensure
       vc_test_request.host = old_request_host
       vc_test_request.request_method = old_request_method
+      vc_test_request.set_header(Rack::RACK_URL_SCHEME, old_request_scheme)
       vc_test_request.path_info = old_request_path_info
       vc_test_request.path_parameters = old_request_path_parameters
       vc_test_request.set_header("action_dispatch.request.query_parameters", old_request_query_parameters)
@@ -223,7 +255,20 @@ module ViewComponent
     #
     # @return [ActionController::Base]
     def vc_test_controller
-      @vc_test_controller ||= __vc_test_helpers_build_controller(Base.test_controller.constantize)
+      @vc_test_controller ||= __vc_test_helpers_build_controller(vc_test_controller_class)
+    end
+
+    # Set the controller used by `render_inline`:
+    #
+    # ```ruby
+    # def vc_test_controller_class
+    #   MyTestController
+    # end
+    # ```
+    def vc_test_controller_class
+      return @__vc_test_controller_class if defined?(@__vc_test_controller_class)
+
+      defined?(ApplicationController) ? ApplicationController : ActionController::Base
     end
 
     # Access the request used by `render_inline`:
@@ -257,11 +302,9 @@ module ViewComponent
 
     def __vc_test_helpers_preview_class
       result = if respond_to?(:described_class)
-        # :nocov:
-        raise "`render_preview` expected a described_class, but it is nil." if described_class.nil?
+        raise ArgumentError.new("`render_preview` expected a described_class, but it is nil.") if described_class.nil?
 
         "#{described_class}Preview"
-        # :nocov:
       else
         self.class.name.gsub("Test", "Preview")
       end
@@ -269,6 +312,5 @@ module ViewComponent
     rescue NameError
       raise NameError, "`render_preview` expected to find #{result}, but it does not exist."
     end
-    # :nocov:
   end
 end
